
import appdirs
import json
import os
from .tools import get_config

config = get_config()


class Artist_db():
    def __init__(self, jsonpath=None, artists=None):
        self._artists = artists if artists is not None else dict()
        self.ignore_list = config.IGNORE_LIST or tuple()
        self.jsonpath = jsonpath
        if jsonpath is not None:
            try:
                if artists is None:
                    self.load()
            except FileNotFoundError:
                pass
            except:
                print("Error when importing the database, creating a fresh "
                      "one...")
                pass

    def _diff_artists(self, artists):
        """
        Compare keys of self._artists with artists

        :param artists: list of artists to compare with
        :type artists: list
        :returns keys: list of artists differents between self._artists and
            artists
        """
        try:
            db_keys = set(self._artists.keys())
        except:
            db_keys = set()
        set_artists = set(artists)
        return db_keys.difference(set_artists).symmetric_difference(
            set_artists.difference(db_keys))

    def _get_fields(self):
        """
        Get the database fields/columns

        :returns fields
        """
        try:
            fields = list(next(iter(self._artists.values())).keys())
        except StopIteration:
            fields = []
        return fields

    def load(self):
        """
        Refresh the artists list from the json file
        """
        with open(self.jsonpath, "r") as f:
            self._artists = json.load(f)

    def save(self):
        """
        Save the artists list into the json file
        """
        new_db = not os.path.exists(self.jsonpath)
        fmode = "a" if new_db else "w"
        try:
            artist_db_dirname = os.path.dirname(self.jsonpath)
            if not os.path.exists(artist_db_dirname):
                os.makedirs(artist_db_dirname)
            with open(self.jsonpath, fmode) as f:
                json.dump(self._artists, f, indent=4)
        except Exception as e:
            print("Error when saving the database")
            print(e)

    def add(self, artists):
        """
        Add artist(s) in the db

        :param artist: artist(s) to add into the db
        :type artists: str or list
        """
        if type(artists) is not str:
            try:
                for a in artists:
                    self.add(a)
                return
            except:
                pass

        if artists not in self._artists.keys():
            self._artists[str(artists)] = {"uploaded": False, }

    def remove(self, artists):
        """
        Remove artist off the db

        :param artist: artist(s) to remove off the db
        :type artists: str or list
        """
        if type(artists) is not str:
            try:
                for a in artists:
                    self.remove(a)
                return
            except:
                pass

        if artists in self._artists.keys():
            self._artists.pop(artists)

    def _artists_grouped_by(self, artists, group_by, fields=None):
        """
        Return artists grouped by a field. The get_artists function would
        already have done the filters

        :param artists: artists to group by
        :type artists: dict
        :param group_by: field use to grouping
        :param fields: get other fields, like musicbrainz id. By default, will
                       just the names will be returned
        """
        artists_grouped = dict()
        for artist, val in artists.items():
            if fields is None:
                artist_insert = artist
            else:
                artist_insert = {field: self._artists[artist][field]
                                 for field in fields if field in
                                 self._artists[artist].keys()}
                artist_insert["name"] = artist

            if val[group_by] in artists_grouped.keys():
                artists_grouped[val[group_by]].append(artist_insert)
            else:
                artists_grouped[val[group_by]] = [artist_insert, ]
        return artists_grouped

    def get_artists(self, fields=None, uploaded=None, group_by=None):
        """
        Get the list of artists name, with optional filter and group by.

        If fields is None, it will return a list of artist names. Otherwise, it
        will return a list of dict, with the select fields as keys, and the
        artist name with the key "name". If the field does not exist for an
        artist, it will not ignored (for this artist).
        If group_by is not None, will return a list of artist names group by
        the wanted field.

        :param fields: fields to select
        :type fields: tuple
        :param uploaded: filter on the uploaded field
        :type uploaded: bool
        :param group_by: group by a field
        :type group_by: str
        :returns artists
        """
        artists = self._artists

        for ignore_artist in self.ignore_list:
            try:
                artists.pop(ignore_artist)
            except KeyError:
                continue

        if uploaded is not None:
            artists = {artist: val for artist, val in self._artists.items()
                       if val["uploaded"] == uploaded}

        if group_by is not None and group_by in self._get_fields():
            return self._artists_grouped_by(artists, group_by, fields)

        if fields is None:
            artist_list = [artist for artist in artists.keys()]
        else:
            artist_list = []
            for artist in artists:
                artist_insert = {field: self._artists[artist][field]
                                 for field in fields if field in
                                 self._artists[artist].keys()}
                artist_insert["name"] = artist
                artist_list.append(artist_insert)

        return artist_list

    def get_mbid(self, artist):
        """
        Get the musicbrainz id of an artist

        :param artist: artist name
        """
        try:
            mbid = self._artists[artist]["mbid"]
        except KeyError:
            mbid = None
        return mbid

    def is_ignored(self, artist):
        """
        Check if artist is ignored or not

        :param artist: artist name
        """
        return artist.lower() in self.ignore_list

    def mark_as_uploaded(self, artist):
        """
        Mark an artist as uploaded
        """
        self._artists[artist]["uploaded"] = True

    def mark_as_non_uploaded(self, artist):
        """
        Mark an artist as non uploaded
        """
        self._artists[artist]["uploaded"] = False

    def set_mbid(self, artist, mbid):
        """
        Update the musicbrainz id of an artist

        :param artist: artist name
        :param mbid: Musicbrainz id
        """
        self._artists[artist]["mbid"] = mbid

    def merge(self, artists):
        """
        Merge artists in the db with a list of artists name sent in parameter

        :param artists: list of artists to compare the db with
        :type artists: list
        :returns (added, deleted): tuple of removed and added artists list
        :rtype: tuple
        """
        a_diff = self._diff_artists(artists)
        added = []
        removed = []
        for a in a_diff:
            if a in self.ignore_list:
                continue
            elif a in self._artists:
                self.remove(a)
                removed.append(a)
            else:
                self.add(a)
                added.append(a)
        return (added, removed)



#!/usr/bin/env python3

import appdirs
import argparse
import os
import sys
from mpd_muspy import _release_name, _version


def parse_args():
    parser = argparse.ArgumentParser(
        description="Sync artists of your MPD database with your MuSpy account"
    )

    parser.add_argument(
        "-c", "--clean",
        help=(
            "drop everything of the database to start syncing from a fresh one"
        ), dest="clean", action="store_true"
    )

    parser.add_argument(
        "--version", action="version",
        version="{} {}".format(_release_name, _version)
    )

    parser.set_defaults(func=sync)

    args = parser.parse_args()
    args.func(parsed_args=args)


def sync(parsed_args):
    check_config_exists()

    from mpd_muspy.sync import run as run_sync
    return run_sync(clean=parsed_args.clean)


def check_config_exists():
    try:
        from mpd_muspy.tools import get_config_path
        get_config_path()
    except FileNotFoundError:
        os.environ["XDG_CONFIG_DIRS"] = "/etc"
        print(
            (
                "Configuration file config.py not found. Please create a "
                "configuration file `config.py` into one of the following "
                "directories: {}"
            ).format(
                ", ".join((
                    appdirs.user_config_dir(_release_name),
                    appdirs.site_config_dir(_release_name)
                ))
            ), file=sys.stderr
        )
        sys.exit(1)


if __name__ == "__main__":
    parse_args()



#!/usr/bin/python
# Author: Anthony Ruhier

import multiprocessing
from .muspy_api import Muspy_api
from .tools import chunks, get_mbid, mpd_get_artists, get_config

config = get_config()
try:
    from config import FULLSYNC
except:
    FULLSYNC = False

# After multiple tests, it appears that this value is the best compromise to
# avoid HTTP error 400 with the musicbrainz api
NB_MULTIPROCESS = 3


def process_task(lst_without_mbid, artists_nb, artist_db, lock, counter,
                 error_nb, muspy_artists, mpdclient):
    """
    Function launched by each process

    Add artists on muspy and marks it in the artists database.
    Tries to get the mbid from the list muspy_artists, because it is very fast.
    If it cannot find it, search on musicbrainz.

    :param lst_without_mbid: list of artists, splited for each process, that
        don't have a musicbrainz id
    :type artists: list
    :param artists_nb: total artists to upload. Different of
        len(lst_without_mbid), here it is the total number of artists of all
        processes
    :type artists_nb: int
    :param artist_db: database of artists, in the shared memory
    :type artist_db: SyncManager.Artist_db()
    :param lock: lock shared between the processes
    :type lock: multiprocessing.Lock
    :param counter: integer in the shared memory
    :type counter: multiprocessing.Value("i")
    :param muspy_artists: list of artists already on the muspy account
    :type muspy_artists: list of dict
    :param mpdclient:
    """
    for artist in lst_without_mbid:
        error = ""
        try:
            mbid = None
            for ma in muspy_artists:
                if ma["name"] == artist:
                    mbid = ma["mbid"]
                    break
            if mbid is None:
                mbid = get_mbid(artist, mpdclient)
            if mbid is not None:
                with lock:
                    artist_db.set_mbid(artist, mbid)
                    artist_db.save()
        except Exception as e:
            error = "Error: " + str(e)
            with lock:
                error_nb.value += 1
        finally:
            with lock:
                counter.value += 1
            print("[", counter.value, "/", artists_nb, "]:", artist.title())
            if error:
                print(error)


def fetch_missing_mbid(artist_db, muspy_artists, mpdclient):
    """
    Initialize the synchronization in several process

    :param artist_db: Artist_db() object in the shared memory
    :type artist_db: SyncManager.Artist_db
    :param muspy_artists: artists already on the muspy account
    :type muspy_artists: list of dict
    :param mpdclient: client for mpd
    :type mpdclient: SyncManager.MPDClient
    """
    # Get all artists name that don't have an musicbrainz id
    lst_without_mbid = [a["name"]
                        for a in artist_db.get_artists(fields=("mbid",))
                        if not ("mbid" in a.keys() and a["mbid"] is not None)]

    manager = multiprocessing.Manager()
    lock = manager.Lock()
    error = manager.Value("i", 0)
    counter = manager.Value("i", 0)
    artists_nb = len(lst_without_mbid)
    artists_nb_by_split = int(artists_nb / NB_MULTIPROCESS)
    pool = multiprocessing.Pool()

    try:
        for l in chunks(lst_without_mbid, artists_nb_by_split):
            pool.apply_async(
                process_task,
                kwds={"lst_without_mbid": l,
                      "artists_nb": artists_nb, "artist_db": artist_db,
                      "lock": lock, "counter": counter, "error_nb": error,
                      "muspy_artists": muspy_artists, "mpdclient": mpdclient}
            )
        pool.close()
        pool.join()
    except KeyboardInterrupt as e:
        pool.terminate()
        pool.join()
        raise e
    return error.value


def update_artists_from_muspy(artist_db, muspy_artists):
    """
    Update the uploaded state of artists from the ones already on the muspy
    account.

    :param artist_db: database of local artists
    :param muspy_artists: list of artists already on the muspy account
    :return remove_of_muspy: list of artists that should be removed of muspy to
                             get a full synchronisation with the local mpd
    """
    local_artists = artist_db.get_artists(fields=("mbid", "uploaded",))
    muspy_mbid_list = {ma["mbid"] for ma in muspy_artists}
    # usefull for fullsync
    uniq_local_artists = set()
    for la in local_artists:
        try:
            uniq_local_artists.add(la["mbid"])
            if la["mbid"] in muspy_mbid_list and la["uploaded"] is False:
                artist_db.mark_as_uploaded(la["name"])
            elif la["mbid"] not in muspy_mbid_list and la["uploaded"] is True:
                artist_db.mark_as_non_uploaded(la["name"])
        except KeyError:
            pass
    artist_db.save()
    if FULLSYNC:
        remove_of_muspy = muspy_mbid_list.difference(uniq_local_artists)
        remove_of_muspy = [(ma["name"], ma["mbid"]) for ma in muspy_artists
                           if ma["mbid"] in remove_of_muspy and
                           not artist_db.is_ignored(ma["name"])]
    else:
        remove_of_muspy = []
    return remove_of_muspy


def presync(artist_db, mpdclient):
    print("Get mpd artists...")
    artists = mpd_get_artists(mpdclient)
    artists_added, artists_removed = artist_db.merge(artists)

    mapi = Muspy_api()
    muspy_artists = mapi.get_artists()

    print("Fetch the missing musicbrainz ids...")
    error = fetch_missing_mbid(artist_db, muspy_artists, mpdclient)
    print()
    if error:
        print("Done with", error, "error(s)\n")
    else:
        print("Done\n")

    # Update the uploaded status of artists in the db with the muspy account
    print("Pre-synchronization with muspy...")
    remove_of_muspy = update_artists_from_muspy(artist_db, muspy_artists)
    artist_db.save()

    non_uploaded_artists = artist_db.get_artists(fields=("mbid",),
                                                 uploaded=False)
    print()
    print(len(non_uploaded_artists), "artist(s) non uploaded on muspy")
    print(len(artists_added), "artist(s) added")
    print(len(artists_removed), "artist(s) removed")

    return non_uploaded_artists, remove_of_muspy



#!/usr/bin/python
# Author: Anthony Ruhier

import appdirs
import os
import mpd
import multiprocessing
from multiprocessing.managers import BaseManager
from . import _release_name
from .artist_db import Artist_db
from .muspy_api import Muspy_api
from .presync import presync
from .tools import chunks, get_config

config = get_config()
from config import ARTISTS_JSON

ARTISTS_JSON = os.path.join(
    appdirs.user_data_dir(_release_name), ARTISTS_JSON
)
NB_MULTIPROCESS = 5


class SyncManager(BaseManager):
    pass


SyncManager.register('Artist_db', Artist_db)
SyncManager.register('MPDClient', mpd.MPDClient)


def process_add_artists(artists, artists_nb, artist_db, lock, error_nb,
                        counter):
    """
    Function launched by each process to add artists on muspy

    Add artists on muspy and marks it in the artists database.

    :param artists: list of artists split for this process
    :type artists: list
    :param artists_nb: total artists to upload. Different of len(artists), here
                       it is the total number of artists of all processes.
    :type artists_nb: int
    :param artist_db: database of artists, in the shared memory
    :type artist_db: SyncManager.Artist_db()
    :param lock: lock shared between the processes
    :type lock: multiprocessing.Lock
    :param counter: integer in the shared memory
    :type counter: multiprocessing.Value("i")
    """
    muspy_api = Muspy_api()
    for artist in artists:
        error = ""
        try:
            if "mbid" in artist.keys():
                muspy_api.add_artist_mbid(artist["mbid"])
                with lock:
                    artist_db.mark_as_uploaded(artist["name"])
                    artist_db.save()
            else:
                error = "Doesn't have a musicbrainz ID"
                with lock:
                    error_nb.value += 1
        except Exception as e:
            error = "Error: " + str(e)
            with lock:
                error_nb.value += 1
        finally:
            with lock:
                counter.value += 1
            print("[", counter.value, "/", artists_nb, "]:",
                  artist["name"].title())
            if error:
                print(error)


def process_del_artists(artists, artists_nb, lock, error_nb, counter):
    """
    Function launched by each process to add artists on muspy

    Add artists on muspy and marks it in the artists database.

    :param artists: list of artists split for this process
    :type artists: list
    :param artists_nb: total artists to upload. Different of len(artists), here
                       it is the total number of artists of all processes.
    :type artists_nb: int
    :param artist_db: database of artists, in the shared memory
    :type artist_db: SyncManager.Artist_db()
    :param lock: lock shared between the processes
    :type lock: multiprocessing.Lock
    :param counter: integer in the shared memory
    :type counter: multiprocessing.Value("i")
    """
    muspy_api = Muspy_api()
    for artist in artists:
        error = ""
        try:
            muspy_api.del_artist_mbid(artist[1])
        except Exception as e:
            error = "Error: " + str(e)
            with lock:
                error_nb.value += 1
        finally:
            with lock:
                counter.value += 1
            print("[", counter.value, "/", artists_nb, "]:",
                  artist[0].title())
            if error:
                print(error)


def start_pool_del(remove_of_muspy):
    """
    Initialize the synchronization in several process to remove of muspy

    :param non_uploaded_artists: list of artists name to upload
    :type non_uploaded_artists: list
    :param artist_db: Artist_db() object in the shared memory
    :type artist_db: SyncManager.Artist_db
    """
    manager = multiprocessing.Manager()
    lock = manager.Lock()
    error = manager.Value("i", 0)
    counter = manager.Value("i", 0)
    artists_nb = len(remove_of_muspy)
    artists_nb_by_split = int(artists_nb / NB_MULTIPROCESS)
    pool = multiprocessing.Pool()

    try:
        for l in chunks(remove_of_muspy, artists_nb_by_split):
            pool.apply_async(
                process_del_artists,
                kwds={"artists": l, "artists_nb": artists_nb,
                      "lock": lock, "error_nb": error,
                      "counter": counter}
            )
        pool.close()
    except Exception as e:
        pool.terminate()
        raise e
    finally:
        pool.join()
    return error.value


def start_pool_add(non_uploaded_artists, artist_db):
    """
    Initialize the synchronization in several process to add artist on muspy

    :param non_uploaded_artists: list of artists name to upload
    :type non_uploaded_artists: list
    :param artist_db: Artist_db() object in the shared memory
    :type artist_db: SyncManager.Artist_db
    """
    manager = multiprocessing.Manager()
    lock = manager.Lock()
    error = manager.Value("i", 0)
    counter = manager.Value("i", 0)
    artists_nb = len(non_uploaded_artists)
    artists_nb_by_split = int(artists_nb / NB_MULTIPROCESS)
    pool = multiprocessing.Pool()

    try:
        for l in chunks(non_uploaded_artists, artists_nb_by_split):
            pool.apply_async(
                process_add_artists,
                kwds={"artists": l, "artists_nb": artists_nb,
                      "artist_db": artist_db, "lock": lock, "error_nb": error,
                      "counter": counter}
            )
        pool.close()
        pool.join()
    except Exception as e:
        pool.terminate()
        pool.join()
        raise e
    return error.value


def run(clean=False):
    """
    Run synchronization. If clean parameter is specified, remove everything in
    the current database, to start on a clean one.

    :param clean: boolean about if starting a clean synchronization (drop the
        db) or not.
    :type clean: boolean
    """
    process_manager = SyncManager()
    process_manager.start()
    if clean:
        artist_db = process_manager.Artist_db(
            jsonpath=ARTISTS_JSON, artists={})
        artist_db.save()
    else:
        artist_db = process_manager.Artist_db(jsonpath=ARTISTS_JSON)
    mpdclient = process_manager.MPDClient()
    try:
        non_uploaded_artists, remove_of_muspy = presync(artist_db, mpdclient)

        print("\n   Start syncing\n =================\n")
        error = start_pool_add(non_uploaded_artists, artist_db)
    except Exception as e:
        artist_db.save()
        raise e

    if len(remove_of_muspy):
        print("\nRemoving of Muspy artists who do not exist in mpd "
              "anymore...\n")
        error += start_pool_del(remove_of_muspy)
    msg = ("Done: " +
           str(len(non_uploaded_artists) + len(remove_of_muspy) -
               len(artist_db.get_artists(uploaded=False))) +
           " artist(s) updated")
    if error:
        msg += " with " + str(error) + " errors"
    print()
    print(msg)
    process_manager.shutdown()




import appdirs
from importlib.machinery import SourceFileLoader
import mpd
import musicbrainzngs
import os

from . import _release_name, _version
from .exceptions import ArtistNotFoundException


def get_config():
    return SourceFileLoader("config", get_config_path()).load_module()


def get_config_path():
    os.environ["XDG_CONFIG_DIRS"] = "/etc"
    CONFIG_DIRS = (
        appdirs.user_config_dir(_release_name),
        appdirs.site_config_dir(_release_name),
    )
    CONFIG_FILENAME = "config.py"

    for d in CONFIG_DIRS:
        config_path = os.path.join(d, CONFIG_FILENAME)
        if os.path.isfile(config_path):
            return config_path

    raise FileNotFoundError


config = get_config()
from config import SERVER, PORT
try:
    from config import USE_ALBUMARTIST
except:
    USE_ALBUMARTIST = False

musicbrainzngs.set_useragent(_release_name, _version)


def chunks(l, n):
    """
    Yield successive n-sized chunks from l.

    :param l: list to split
    :param n: number of elements wanted in each list split
    """
    n = 1 if n <= 0 else n
    for i in range(0, len(l), n):
        yield l[i:i+n]


def del_chars_from_string(s, chars_to_del):
    """
    Delete characters from list

    :param s: string to clean
    :param chars_to_del: characters to delete in string
    """
    if type(chars_to_del) != "str":
        for c in chars_to_del:
            s = s.replace(c, "")
    else:
        s = s.replace(chars_to_del, "")
    return s


def mpd_get_artists(mpdclient):
    """
    Get artists from MPD

    :param mpdclient: connection with MPD
    :type mpdclient: mpd.MPDClient()
    """
    mpdclient.connect(SERVER, PORT)
    tag_field = "albumartist" if USE_ALBUMARTIST else "artist"
    try:
        artists = set(str(artist).lower()
                      for artist in mpdclient.list(tag_field) if len(artist))
    except mpd.ConnectionError:
        mpdclient.connect(SERVER, PORT)
        artists = set(str(artist).lower()
                      for artist in mpdclient.list(tag_field) if len(artist))
    return artists


def get_mpd_albums(artist, mpdclient):
    """
    Get list of albums in the mpd database for an artist

    :param artist: artist name to filter
    :param mpdclient: connector with the mpd server
    :type mpdclient: mpd.MPDClient()
    """
    try:
        mpdclient.status()
    except mpd.ConnectionError:
        mpdclient.connect(SERVER, PORT)
    # The mpd module is using case sensitive filters in list(). Artist has to
    # be spelled correctly
    tag_field = "albumartist" if USE_ALBUMARTIST else "artist"
    try:
        artist_cs = mpdclient.search(tag_field, artist)[0]["artist"]
    except IndexError:
        raise ArtistNotFoundException("Artist is not in the mpd database")
    return mpdclient.list("album", tag_field, artist_cs)


def get_mbid(artist, mpdclient):
    """
    Get the musicbrainz id of an artist

    Search the artist id from the album we have in the mpd database to be
    almost sure the result is good.

    :param artist: artist name to get the id
    """
    ignore_chars = ["/", "\\", "!", "?"]
    LIMIT_NB_ARTIST = 15
    result = musicbrainzngs.search_artists(
        del_chars_from_string(artist, ignore_chars),
        LIMIT_NB_ARTIST)
    if result["artist-count"] == 0:
        raise ArtistNotFoundException("Artist not found")
    artists_prop = [a["id"] for a in result["artist-list"]]
    if result["artist-count"] == 1:
        return artists_prop[0]

    # Tries to get the artist id of one of our album of this artist
    albums = get_mpd_albums(artist, mpdclient)
    # We don't want to test all choices returned by musicbrainz for an album,
    # so we will keep only the LIMIT_NB_ALBUM'th first ones.
    LIMIT_NB_ALBUM = 10
    for album in albums:
        try:
            result = musicbrainzngs.search_releases(
                del_chars_from_string(album, ignore_chars),
                limit=LIMIT_NB_ALBUM)["release-list"]
            for i in result:
                artist_id = i["artist-credit"][0]["artist"]["id"]
                if artist_id in artists_prop:
                    return artist_id
        except:
            pass
    return artists_prop[0]



#!/usr/bin/python
# Author: Anthony Ruhier

import urllib.request
import mpd
import requests
import config
from .exceptions import ArtistNotFoundException
from .tools import get_mbid, get_config

config = get_config()
from config import MUSPY_ADDR, MUSPY_USERNAME, MUSPY_PASSWORD, MUSPY_ID


class Muspy_api():
    #: URL to target the muspy api
    _muspy_api_url = MUSPY_ADDR

    #: custom opener with authentication
    _urlopener = None

    #: muspy username
    username = None

    #: muspy password
    password = None

    #: muspy user id
    user_id = None

    #: MPDClient object
    _mpdclient = None

    def __init__(self, username=MUSPY_USERNAME, password=MUSPY_PASSWORD,
                 user_id=MUSPY_ID, *args, **kwargs):
        self.username = username
        self.password = password
        self.user_id = user_id
        self._mpdclient = mpd.MPDClient()
        try:
            self._ssl_verify = not config.MUSPY_FORCE_SSL_ACCEPT
            if not self._ssl_verify:
                requests.packages.urllib3.disable_warnings()
        except:
            self._ssl_verify = True

    def add_artist_mbid(self, mbid):
        """
        Add artist by its MusicBrainz id to the muspy account

        :param mbid: MusicBrainz id of the artist
        """
        try:
            requests.put(
                urllib.request.urljoin(
                    self._muspy_api_url,
                    "artists/" + self.user_id + "/" + str(mbid)
                ),
                auth=(MUSPY_USERNAME, MUSPY_PASSWORD),
                verify=self._ssl_verify,
            )
        except requests.HTTPError:
            raise ArtistNotFoundException("Artist not found")

    def del_artist_mbid(self, mbid):
        """
        Delete artist by its MusicBrainz id of the muspy account

        :param mbid: MusicBrainz id of the artist
        """
        try:
            requests.delete(
                urllib.request.urljoin(
                    self._muspy_api_url,
                    "artists/" + self.user_id + "/" + str(mbid)
                ),
                auth=(MUSPY_USERNAME, MUSPY_PASSWORD),
                verify=self._ssl_verify,
            )
        except requests.HTTPError:
            raise ArtistNotFoundException(
                "Artist is not indexed in the Muspy account"
            )

    def add_artist(self, artist):
        """
        Add artist by its name to the muspy account

        :param artist: Artist name to add
        """
        return self.add_artist_mbid(get_mbid(artist, self._mpdclient))

    def del_artist(self, artist):
        """
        Delete artist by its name from the muspy account

        :param artist: Artist name to add
        """
        return self.del_artist_mbid(get_mbid(artist, self._mpdclient))

    def get_artists(self):
        """
        Get artists followed by the user

        MuSpy returns a list of dicts of this form: [{
            'disambiguation':,
            'mbid': musicbrainz id,
            'name': artist name,
            'sort_name':,
        }, ...]

        :returns artists: list of dicts
        """
        r = requests.get(
            urllib.request.urljoin(
                self._muspy_api_url,
                "artists/" + self.user_id
            ),
            auth=(MUSPY_USERNAME, MUSPY_PASSWORD),
            verify=self._ssl_verify,
        )
        return [{"name": a["name"].lower(), "mbid": a["mbid"]}
                for a in r.json()]



#!/usr/bin/env python3

"""
Sync artists of your MPD database with your MuSpy account.
See:
    https://github.com/Anthony25/mpd-muspy
"""

from setuptools import setup

setup(
    name="mpd-muspy",
    version="1.0.1",

    description="Sync artists of your MPD database with your MuSpy account",

    url="https://github.com/Anthony25/mpd-muspy",
    author="Anthony25 <Anthony Ruhier>",
    author_email="anthony.ruhier@gmail.com",

    license="Simplified BSD",

    classifiers=[
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: BSD License",
    ],

    keywords="mpd",
    packages=["mpd_muspy", ],
    install_requires=[
        "appdirs", "argparse", "python-mpd2", "musicbrainzngs", "requests",
        "urllib3"
    ],
    entry_points={
        'console_scripts': [
            'mpd-muspy = mpd_muspy.__main__:parse_args',
        ],
    }
)
